from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.coordinates import SkyCoord, Angle
from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

IMAGING_DEFAULT_LIMIT = 100000


class ImagingClass:
    """
    The imaging class.
    Use to retrieve data from the imaging module.

    **An imaging instance is already provided, to use it :**

    >>> from dace_query.imaging import Imaging

    """

    __ACCEPTED_FILE_TYPES = ['ns', 'snr', 'dl', 'hc', 'pa', 'master', 'all']
    __IMAGING_FILENAMES = {'NS': 'ns.fits',
                           'SNR': 'snr.fits',
                           'DL': 'dl.rdb',
                           'PA': 'PA.rdb',
                           'MASTER': 'master.fits',
                           'HC': 'hc.fits'}

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable imaging obejct which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.imaging import ImagingClass
        >>> imaging_class = ImagingClass()
        """
        self.__OBS_API = 'obs-webapp'

        self.log = logging.getLogger("Imaging")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = IMAGING_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the imaging database to retrieve data in the chosen format.

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.imaging import Imaging
        >>> values = Imaging.query_database()

        """

        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OBS_API,
                endpoint='observation/search/imaging',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)}
            ), output_format=output_format)

    def query_region(self,
                     sky_coord: SkyCoord,
                     angle: Angle,
                     filters: Optional[dict] = None,
                     output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query a region, based on SkyCoord and Angle objects, in the imaging database and retrieve data in the chosen
        format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param sky_coord: Sky coordinates object from the astropy module
        :type sky_coord: SkyCoord
        :param angle: Angle object from the astropy module
        :type angle: Angle
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.imaging import Imaging
        >>> from astropy.coordinates import SkyCoord, Angle
        >>> from astropy import units as u
        >>> sky_coord, angle = SkyCoord("02:12:20.6774","-46:48:58.9566", unit=(u.hourangle, u.deg)), Angle('0.045d')
        >>> values = Imaging.query_region(sky_coord=sky_coord, angle=angle)
        """

        coordinate_filter_dict = self.dace.transform_coordinates_to_dict(sky_coord, angle)
        filters_with_coordinates = {}
        if filters is not None:
            filters_with_coordinates.update(filters)
        filters_with_coordinates.update(coordinate_filter_dict)
        return self.query_database(filters=filters_with_coordinates, output_format=output_format)

    def download(self,
                 file_type: str,
                 filters: Optional[dict] = None,
                 output_directory: Optional[str] = None,
                 output_filename: Optional[str] = None) -> None:
        """
        Download specified file type from the imaging module.

        Available file types are [ 'ns', 'snr', 'dl', 'hc', 'pa', 'master', 'all' ].

        * **ns :** non saturated
        * **snr :** signal-to-noise ratio
        * **dl :** detection limit (.rdb file)
        * **hc :** high contrast
        * **pa :** Parallactic angle (.rdb file)
        * **master :** master
        * **all :** all files

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).


        :param file_type: File type
        :type file_type: str
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.imaging import Imaging
        >>> filters_to_use = {'file_rootpath':{'contains':'sphere/SPHERE-DRS/DRS-1.0/reduced/2018-08-19/SPHERE_IRDIS.2018-08-19T07:03:54.679_H2.fits' }}
        >>> # Imaging.download(file_type='ns', filters=filters_to_use, output_directory='/tmp', output_filename='files.tar.gz')

        """

        if file_type not in self.__ACCEPTED_FILE_TYPES:
            raise ValueError('file_type must be one of these values : ' + ','.join(self.__ACCEPTED_FILE_TYPES))
        if filters is None:
            filters = {}

        imaging_data = self.query_database(filters=filters, output_format='dict')
        files = imaging_data['file_rootpath']
        download_response = self.dace.request_post(
            api_name=self.__OBS_API,
            endpoint='download/prepare/imaging',
            data=json.dumps({
                'fileType': file_type,
                'files': files
            })
        )
        if not download_response:
            return None
        download_id = download_response['values'][0]
        self.dace.persist_file_on_disk(
            api_name=self.__OBS_API,
            obs_type='imaging',
            download_id=download_id,
            output_directory=output_directory,
            output_filename=output_filename
        )

    def get_image(self,
                  fits_file: str,
                  file_type: str,
                  output_directory: Optional[str] = None,
                  output_filename: Optional[str] = None) -> None:
        """
        Download a certain fits imaging files specified by named arguments.

        Available file types are [ 'ns', 'snr', 'dl', 'hc', 'pa', 'master', 'all' ].

        * **ns :** non saturated
        * **snr :** signal-to-noise ratio
        * **dl :** detection limit (.rdb file)
        * **hc :** high contrast
        * **pa :** Parallactic angle (.rdb file)
        * **master :** master
        * **all :** all files

        :param fits_file: The root fits file to download
        :type fits_file: str
        :param file_type: The file type to download
        :type file_type: str
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.imaging import Imaging
        >>> fits_file_to_download, file_type = 'sphere/SPHERE-DRS/DRS-1.0/reduced/2018-08-19/SPHERE_IRDIS.2018-08-19T07:03:54.679_H2.fits', 'hc'
        >>> # Imaging.get_image(fits_file=fits_file_to_download, file_type=file_type, output_directory='/tmp', output_filename='imaging.fits')
        """

        file_type = str(file_type).upper()
        # url = '/imaging/file?filepath=' + fits_file + '&filterType=' + file_type
        self.dace.download_file(
            api_name=self.__OBS_API,
            endpoint='observation/imaging/file',
            params={
                'filepath': fits_file,
                'filterType': file_type
            },
            output_directory=output_directory,
            output_filename=output_filename
        )


Imaging: ImagingClass = ImagingClass()
"""Imaging instance"""



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

MOLECULE_DEFAULT_LIMIT = 10000


class MoleculeClass:
    """
    The molecule class.
    Use to retrieve data from the molecule module.

    **A molecule instance is already provided, to use it :**

    >>> from dace_query.opacity import Molecule

    """

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable molecule object which uses a specified instance.

        :param dace_instance: a dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.opacity import MoleculeClass
        >>> molecule_instance = MoleculeClass()
        """

        self.__OPACITY_API = 'opa-webapp'

        # Logging configuration
        self.log = logging.getLogger("Opacity")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = MOLECULE_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the molecule opacity database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.opacity import Molecule
        >>> values = Molecule.query_database()
        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OPACITY_API,
                endpoint='molecule/search',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            ), output_format=output_format
        )

    def download(self, isotopologue: str, line_list: str, version: str,
                 temperature_boundaries: tuple[int, int],
                 pressure_boundaries: tuple[float, float],
                 output_directory: Optional[str] = None,
                 output_filename: Optional[str] = None) -> None:
        """
        Download data of a specified molecule from the opacity module.

        :param isotopologue: The isotopologue to retrieve data from
        :type isotopologue: str
        :param line_list: The line list / data source
        :type line_list: str:
        :param version: The version
        :type version: str
        :param temperature_boundaries: The temperature boundaries
        :type temperature_boundaries: tuple[int, int]
        :param pressure_boundaries: The pressure boundaries
        :type pressure_boundaries: tuple[float, float]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.opacity import Molecule
        >>> # Molecule.download('1H2-16O', 'POKAZATEL', 1.0, (2500, 2600), (2.5, 3), output_directory='/tmp', output_filename='test_molecule.tar.gz')
        """

        self.dace.download_file(
            api_name=self.__OPACITY_API,
            endpoint=f'molecule/download/{isotopologue}/{line_list}',
            params={
                'tMin': str(temperature_boundaries[0]),
                'tMax': str(temperature_boundaries[1]),
                'pMinExp': str(pressure_boundaries[0]),
                'pMaxExp': str(pressure_boundaries[1]),
                'version': str(version)
            },
            output_directory=output_directory,
            output_filename=output_filename
        )

    def get_data(self,
                 isotopologue: str,
                 line_list: str,
                 version: str,
                 temperature: int,
                 pressure_exponent: float,
                 output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Retrieve data from an isotopologue in the chosen format.

        All available formats are defined in this section (see :doc:`output_format`).


        :param isotopologue: The isotopologue to retrieve data from
        :type isotopologue: str
        :param line_list: The line list
        :type line_list: str
        :param version: The version
        :param temperature: The temperature
        :param pressure_exponent: The pressure exponent
        :param output_format: Type of data returns
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.opacity import Molecule
        >>> values = Molecule.get_data('1H2-16O', 'POKAZATEL', 1.0, 300, -1.33)
        """
        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OPACITY_API,
                endpoint=f'molecule/data/{isotopologue}/{line_list}',
                params={
                    'temperature': str(temperature),
                    'pressureExp': str(pressure_exponent),
                    'version': str(version)
                }
            ), output_format=output_format
        )

    def get_high_resolution_data(self,
                                 isotopologue: str,
                                 line_list: str,
                                 version: str,
                                 temperature: int,
                                 pressure_exponent: float,
                                 wavenumber_boundaries: tuple[float, float],
                                 output_format: Optional[str] = None) -> Union[
        dict[str, ndarray], DataFrame, Table, dict]:
        """
        Retrieve high resolution data from a molecule in the chosen format.

        All available formats are defined in this section (see :doc:`output_format`).

        :param isotopologue: The isotopologue to retrieve high resolution data from
        :type isotopologue: str
        :param line_list: The line list
        :type line_list: str
        :param version: The version
        :type version: str
        :param temperature: The temperature
        :param pressure_exponent: The pressure exponent
        :param wavenumber_boundaries: The range min and max to extract from the binary high resolution file
        :type wavenumber_boundaries: tuple[float, float]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.opacity import Molecule
        >>> values =  Molecule.get_high_resolution_data('1H2-16O', 'POKAZATEL', 1.0, 300, -1.33, (1.01, 3.02))
        """
        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OPACITY_API,
                endpoint=f'molecule/highresolutiondata/{isotopologue}/{line_list}',
                params={
                    'temperature': str(temperature),
                    'pressureExp': str(pressure_exponent),
                    'wavenumberStart': str(wavenumber_boundaries[0]),
                    'wavenumberEnd': str(wavenumber_boundaries[1]),
                    'version': str(version)
                }
            ), output_format=output_format
        )

    def interpolate(self,
                    isotopologue: str,
                    line_list: str,
                    version: str,
                    interpol_temperatures: list,
                    interpol_pressures: list,
                    output_directory: Optional[str] = None,
                    output_filename: Optional[str] = None):
        """
        Compute interpolation for an isotopologue.

        :param isotopologue: The isotopologue
        :type isotopologue: str
        :param line_list: The line list / data source
        :type line_list: str
        :param version: The version
        :type version: str
        :param interpol_temperatures: The interpol temperatures
        :type interpol_temperatures: list
        :param interpol_pressures: The interpol pressures
        :type interpol_pressures: list
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.opacity import Molecule
        >>> # Molecule.interpolate('1H2-16O', 'POKAZATEL', 1.0, [110], [0.4], output_directory='/tmp', output_filename='opacity_molecule_interpolate.tar.gz')

        """
        download_response = self.dace.request_post(
            api_name=self.__OPACITY_API,
            endpoint=f'molecule/interpolate/{isotopologue}/{line_list}',
            params={
                'version': str(version),
            },
            data=json.dumps({
                'interpol_temperatures': interpol_temperatures,
                'interpol_pressures': interpol_pressures
            })
        )
        if not download_response:
            return None
        download_id = download_response['values'][0]
        self.dace.download_file(
            api_name=self.__OPACITY_API,
            endpoint=f'molecule/interpolate/{download_id}',
            output_directory=output_directory,
            output_filename=output_filename
        )


Molecule: MoleculeClass = MoleculeClass()
"""Molecule instance"""



from __future__ import annotations

import json
import logging
from typing import Optional
from typing import Union

from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

ASTROMETRY_DEFAULT_LIMIT = 10000


class AstrometryClass:
    """
    The astrometry class.
    Use to retrieve data from the astrometry module.

    **An astrometry instance is already provided, to use it :**

    >>> from dace_query.astrometry import Astrometry

    """

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable astrometry object which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.astrometry import AstrometryClass
        >>> astrometry_instance = AstrometryClass()

        """

        # Logging configuration
        self.__OBSERVATION_API = 'obs-webapp'
        self.log = logging.getLogger("Astrometry")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = ASTROMETRY_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) \
            -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the astrometry database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.astrometry import Astrometry
        >>> values = Astrometry.query_database()
        """

        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OBSERVATION_API,
                endpoint='observation/search/astrometry',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }), output_format=output_format)

    def get_gaia_timeseries(self,
                            target: str,
                            output_format: Optional[str] = None) \
            -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Get timeseries from Gaia astrometry for the specified target.

        All available formats are defined in this section (see :doc:`output_format`).

        :param target: The target name to retrieve astrometry data from
        :type target: str
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.astrometry import Astrometry
        >>> target_to_search = 'your-target'
        >>> values = Astrometry.get_gaia_timeseries(target=target_to_search)

        """

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OBSERVATION_API,
                endpoint=f'observation/astrometry/{target}'),
            output_format=output_format)


Astrometry: AstrometryClass = AstrometryClass()
"""
Astrometry instance
"""



from __future__ import annotations

import json
import logging
from typing import Optional

from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass
from dace_query.dace import NoDataException
from dace_query.spectroscopy import Spectroscopy

SUN_DEFAULT_LIMIT = 200000


class SunClass:
    """
    The sun class.
    Use to retrieve data from the sun module.

    **A sun instance is already provided, to use it :**

    >>> from dace_query.sun import Sun

    """

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable sun object which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.sun import SunClass
        >>> sun_instance = SunClass()

        """
        # Logging configuration
        self.__OBS_API = 'obs-webapp'
        self.log = logging.getLogger("Sun")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = SUN_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None):
        """
        Query the sun database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.sun import Sun
        >>> values = Sun.query_database()

        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OBS_API,
                endpoint='sun/search',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            ), output_format=output_format
        )

    def get_timeseries(self, output_format: Optional[str] = None):
        """
        Get all sun timeseries.

        All available formats are defined in this section (see :doc:`output_format`).

        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format

        >>> from dace_query.sun import Sun
        >>> values = Sun.get_timeseries()
        """
        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OBS_API,
                endpoint='sun/radialVelocities',
            ), output_format=output_format
        )

    def download(self,
                 file_type: str,
                 filters: Optional[dict] = None,
                 output_directory: Optional[str] = None,
                 output_filename: Optional[str] = None) -> None:
        """
        Download Sun spectroscopy products (S1D, S2D, ...).

        Available file types are [ 's1d', 's2d', 'ccf', 'bis', 'all' ].

        :param file_type: The type of files to download
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.sun import Sun
        >>> filters_to_use = {'file_rootpath': {'contains': ['r.HARPN.2016-01-03T15-36-20.496.fits']}}
        >>> # Sun.download('s1d', filters=filters_to_use, output_directory='/tmp', output_filename='sun_spectroscopy_data.tar.gz')
        """

        if file_type not in Spectroscopy.ACCEPTED_FILE_TYPES:
            raise ValueError('file_type must be one of these values : ' + ','.join(Spectroscopy.ACCEPTED_FILE_TYPES))
        if filters is None:
            filters = {}

        sun_spectroscopy_data = self.query_database(filters=filters, output_format='dict')
        files = sun_spectroscopy_data['file_rootpath']
        download_response = self.dace.request_post(
            api_name=self.__OBS_API,
            endpoint='download/prepare/sun',
            data=json.dumps({
                'fileType': file_type,
                'files': files
            })

        )
        if not download_response:
            return None
        download_id = download_response['values'][0]
        self.dace.persist_file_on_disk(
            api_name=self.__OBS_API,
            obs_type='sun',
            download_id=download_id,
            output_directory=output_directory,
            output_filename=output_filename
        )

    def download_files(self,
                       file_type: Optional[str] = 's1d',
                       files: Optional[list[str]] = None,
                       output_directory: Optional[str] = None,
                       output_filename: Optional[str] = None) -> None:
        """
        Download reduction products specified in argument for the list of raw files specified and save it locally.

        Available file types are [ 's1d', 's2d', 'ccf', 'bis', 'all' ].

        :param file_type: The type of files to download
        :type file_type: Optional[str]
        :param files: The raw files
        :type files: list[str]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.sun import Sun
        >>> files_to_retrieve = ['harpn/DRS-2.3.5/reduced/2016-01-03/r.HARPN.2016-01-03T15-36-20.496.fits']
        >>> # Sun.download_files('s1d', files=files_to_retrieve, output_directory='/tmp', output_filename='files.tar.gz')

        """
        if files is None:
            raise NoDataException

        files = list(map(lambda file: f'{file}.fits' if not file.endswith('.fits') else file, files))

        # files = [file + '.fits' for file in files if '.fits' not in file]

        download_response = self.dace.request_post(
            api_name=self.__OBS_API,
            endpoint='download/prepare/sun',
            data=json.dumps({
                'fileType': file_type,
                'files': files
            })
        )
        if not download_response:
            return None
        download_id = download_response['values'][0]
        self.dace.persist_file_on_disk(
            api_name=self.__OBS_API,
            obs_type='sun',
            download_id=download_id,
            output_directory=output_directory,
            output_filename=output_filename
        )

    def download_public_release_all(self,
                                    year: str,
                                    month: str,
                                    output_directory: Optional[str] = None,
                                    output_filename: Optional[str] = None) -> None:
        """
        Download public sun data of year and month specified in arguments.

        :param year: The year for sun data
        :type year: str
        :param month: The month for sun data
        :type month: str
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.sun import Sun
        >>> # Sun.download_public_release_all('2015','12', output_directory='/tmp', output_filename='release_all_2015-12.tar.gz')

        """
        year_and_month = str(year) + '-' + str(month)
        self.dace.download_file(
            api_name=self.__OBS_API,
            endpoint=f'sun/download/release/all/{year_and_month}',
            output_directory=output_directory,
            output_filename=output_filename
        )

    def download_public_release_ccf(self,
                                    year: str,
                                    output_directory: Optional[str] = None,
                                    output_filename: Optional[str] = None) -> None:
        """
        Download public ccf data realease of year specified in argument.

        :param year: The year for the ccf data
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.sun import Sun
        >>> # Sun.download_public_release_ccf('2015', output_directory='/tmp', output_filename='cff.tar.gz')
        """
        self.dace.download_file(
            api_name=self.__OBS_API,
            endpoint=f'sun/download/release/ccf/{year}',
            output_directory=output_directory,
            output_filename=output_filename
        )

    def download_public_release_timeseries(self,
                                           period: Optional[str] = '2015-2018',
                                           output_directory: Optional[str] = None,
                                           output_filename: Optional[str] = None) -> None:
        """
        Download public timeseries data release for a specified period and save it locally.

        The only available period is '2015-2018'.

        :param period: The period
        :type period: Optional[str]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.sun import Sun
        >>> # Sun.download_public_release_timeseries(output_directory='/tmp', output_filename='public_release_timeseries.rdb')
        """
        self.dace.download_file(
            api_name=self.__OBS_API,
            endpoint=f'sun/download/release/timeseries/{period}',
            output_directory=output_directory,
            output_filename=output_filename
        )


Sun: SunClass = SunClass()
"""Sun instance"""



import sys

major, minor, micro = sys.version_info.major, sys.version_info.minor, sys.version_info.micro

# Support, will be deprecated
if major >= 3 and minor >= 8:
    from importlib.metadata import metadata, PackageNotFoundError
else:
    from importlib_metadata import metadata, PackageNotFoundError

# Define the package name
__pkg_name__ = __package__  # Can be replaced by __package__ if root folder and package are the same

try:
    __title__ = metadata(__pkg_name__)['name']
    __version__ = metadata(__pkg_name__)['version']
except PackageNotFoundError:
    __title__ = 'dev'
    __version__ = '0.0.0-dev'

__py_version__ = '.'.join(map(str, [major, minor, micro]))



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.coordinates import SkyCoord, Angle
from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

TESS_DEFAULT_LIMIT = 10000


class TessClass:
    """
    The tess class.
    Use to retrieve data from the tess module.

    **A tess instance is already provided, to uses it :**

    >>> from dace_query.tess import Tess
    """

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable tess instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.tess import TessClass
        >>> tess_instance = TessClass()
        """
        self.__TESS_API = 'tess-webapp'

        # Logging configuration
        self.log = logging.getLogger("Tess")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = TESS_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the tess database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: The desired data in the chosen output format
        :type output_format: Optional[str]
        :return: A dict containing lists of values for each visit
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.tess import Tess
        >>> values = Tess.query_database()
        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__TESS_API,
                endpoint='search',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            ), output_format=output_format
        )

    def query_region(self,
                     sky_coord: SkyCoord,
                     angle: Angle,
                     limit: Optional[int] = TESS_DEFAULT_LIMIT,
                     filters: Optional[dict] = None,
                     output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query a region, based on SkyCoord and Angle objects, in the tess database and retrieve data in the chosen
        format.

        Filters can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param sky_coord: Sky coordinates object from the astropy module
        :type sky_coord: SkyCoord
        :param angle: Angle object from the astropy module
        :type angle: Angle
        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.tess import Tess
        >>> from astropy.coordinates import SkyCoord, Angle
        >>> sky, a = SkyCoord('13h50m24s', '-60d21m11s', frame='icrs'), Angle('0.005d')
        >>> values = Tess.query_region(sky_coord=sky, angle=a)

        """

        coordinate_filter_dict = self.dace.transform_coordinates_to_dict(sky_coord, angle)
        filters_with_coordinates = {}
        if filters is not None:
            filters_with_coordinates.update(filters)
        filters_with_coordinates.update(coordinate_filter_dict)
        return self.query_database(limit=limit, filters=filters_with_coordinates, output_format=output_format)

    def get_flux(self,
                 target: str,
                 flux_type: Optional[dict] = "raw_flux",
                 output_format: Optional[str] = None) \
            -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Retrieve tess photometry data for a specified target in the chosen format.

        All available formats are defined in this section (see :doc:`output_format`).

        Avalable flux types are [ 'raw_flux', 'corr_flux' ].

        :param target: The target to retrieve data from.
        :type target: str
        :param flux_type: The flux type to use
        :type flux_type: Optional[str]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.tess import Tess
        >>> target_to_search = 'TIC381400181'
        >>> values = Tess.get_flux(target=target_to_search)
        """
        options: dict = {
            "fluxType": flux_type,
            "computeModel": {
                "frequency": 48
            },
            "model": {
                "keplerians": {},
                "star": {
                    "RHO_RHOSUN": 1,
                    "LIMBDARK": "quadratic",
                    "LIMBDARKU0": 0.1,
                    "LIMBDARKU1": 0.3
                },
                "offsets": {}
            }

        }
        res = self.dace.request_post(
            api_name=self.__TESS_API,
            endpoint=f'flux/{target}',
            json_data=json.loads(json.dumps(options))
        )
        formatted_res = {}
        for key in res:
            formatted_res[key] = self.dace.transform_to_format(res[key], output_format=output_format)

        return formatted_res


Tess: TessClass = TessClass()
"""Tess instance"""



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

LOSSY_DEFAULT_LIMIT = 10000


class LossyClass:
    """
    The lossy class.
    Use to retrieve data from the lossy module.

    **A lossy instance is already provided, to use it :**

    >>> from dace_query.lossy import Lossy

    """

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable lossy object which uses a specified dace instance.

        :param dace_instance: Optional[DaceClass]

        >>> from dace_query.lossy import LossyClass
        >>> lossy_instance = LossyClass()

        """
        self.__LOSSY_API = 'lossy-webapp'
        # Logging configuration
        self.log = logging.getLogger("Population")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = LOSSY_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the lossy database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.lossy import Lossy
        >>> values = Lossy.query_database()
        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__LOSSY_API,
                endpoint='sample/search',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }), output_format=output_format)

    def get_sample(self,
                   sample_id: str,
                   output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Get the data for a specific sample.

        :param sample_id: The sample to retrieve data from.
        :type sample_id: str
        :param output_format: Type of data returns
        :type output_format: dict[str, ndarray] or DataFrame or Table or dict
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.lossy import Lossy
        >>> sample_id_to_retrieve = 'SAMPLE_Ice_LN2_11_20140221_000'
        >>> values = Lossy.get_sample(sample_id=sample_id_to_retrieve)
        """

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__LOSSY_API,
                endpoint=f'sample/{sample_id}'),
            output_format=output_format)


Lossy: LossyClass = LossyClass()
"""Lossy instance"""



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.coordinates import SkyCoord, Angle
from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass
from dace_query.dace import NoDataException

SPECTROSCOPY_DEFAULT_LIMIT = 10000


class SpectroscopyClass:
    """
    The spectroscopy class.
    Use to retrieve data from the spectroscopy module.

    **A spectroscopy instance is already provided, to use it:**

    >>> from dace_query.spectroscopy import Spectroscopy

    """
    ACCEPTED_FILE_TYPES = ['s1d', 's2d', 'ccf', 'bis', 'all']
    __OBS_API = 'obs-webapp'

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable spectroscopy object which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.spectroscopy import SpectroscopyClass
        >>> spectroscopy_instance = SpectroscopyClass()
        """
        # Logging configuration
        self.log = logging.getLogger("Spectroscopy")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = SPECTROSCOPY_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the spectroscopy database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.spectroscopy import Spectroscopy
        >>> values = Spectroscopy.query_database()

        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OBS_API,
                endpoint='observation/search/spectroscopy',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            ), output_format=output_format
        )

    def query_region(self,
                     sky_coord: SkyCoord,
                     angle: Angle,
                     limit: Optional[int] = SPECTROSCOPY_DEFAULT_LIMIT,
                     filters: Optional[dict] = None,
                     output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query a region, based on SkyCoord and Angle objects, in the spectroscopy database and retrieve data in the chosen format.

        Filters can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param sky_coord: Sky coordinates object from the astropy module
        :type sky_coord: SkyCoord
        :param angle: Angle object from the astropy module
        :type angle: Angle
        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.spectroscopy import Spectroscopy
        >>> from astropy.coordinates import SkyCoord, Angle
        >>> sky_coord, angle = SkyCoord("23h13m16s", "+57d10m06s", frame='icrs'), Angle('0.045d')
        >>> values = Spectroscopy.query_region(sky_coord=sky_coord, angle=angle)
        """
        coordinate_filter_dict = self.dace.transform_coordinates_to_dict(sky_coord, angle)
        filters_with_coordinates = {}
        if filters is not None:
            filters_with_coordinates.update(filters)
        filters_with_coordinates.update(coordinate_filter_dict)
        return self.query_database(limit=limit, filters=filters_with_coordinates, output_format=output_format)

    def download(self,
                 file_type: str,
                 filters: Optional[dict] = None,
                 output_directory: Optional[str] = None,
                 output_filename: Optional[str] = None):
        """
        Download Spectroscopy products (S1D, S2D, ...) and save it locally depending on the specified arguments.

        File type available are [ 's1d', 's2d', 'ccf', 'bis', 'guidance', 'all' ].

        Filters can be applied to the query via named arguments (see :doc:`query_options`).

        :param file_type: The type of files to download
        :type file_type: str
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]

        >>> from dace_query.spectroscopy import Spectroscopy
        >>> filters_to_use = {'file_rootpath': {'contains':['HARPS.2010-04-04T03:38:51.386.fits']}}
        >>> # Spectroscopy.download('s1d', filters=filters_to_use, output_filename='files.tar.gz')
        """
        if file_type not in self.ACCEPTED_FILE_TYPES:
            raise ValueError('file_type must be one of these values : ' + ','.join(self.ACCEPTED_FILE_TYPES))
        if filters is None:
            filters = {}

        spectroscopy_data = self.query_database(filters=filters, output_format='dict')
        files = spectroscopy_data['file_rootpath']
        download_response = self.dace.request_post(
            api_name=self.__OBS_API,
            endpoint='download/prepare/spectroscopy',
            data=json.dumps({
                'fileType': file_type,
                'files': files
            })
        )
        if not download_response:
            return None
        download_id = download_response['values'][0]

        self.dace.persist_file_on_disk(
            api_name=self.__OBS_API,
            obs_type='spectroscopy',
            download_id=download_id,
            output_directory=output_directory,
            output_filename=output_filename
        )

    def download_files(self,
                       files: list,
                       file_type: Optional[str] = 'all',
                       output_directory: Optional[str] = None,
                       output_filename: Optional[str] = None):
        """
        Download reduction products specified in argument for the list of raw files specified and save it locally.

        File type available are ['s1d', 's2d', 'ccf', 'bis', 'guidance', 'all']

        :param files: The raw files
        :param file_type: The type of files to download
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.spectroscopy import Spectroscopy
        >>> files_to_download = ['harps/DRS-3.5/reduced/2019-07-05/HARPS.2019-07-06T04:00:00.323.fits']
        >>> # Spectroscopy.download_files(files=files_to_download, file_type='all')
        """

        if files is None:
            raise NoDataException

        files = list(map(lambda file: f'{file}.fits' if not file.endswith('.fits') else file, files))
        # files = [file + '.fits' for file in files if '.fits' not in file]
        download_response = self.dace.request_post(
            api_name=self.__OBS_API,
            endpoint='download/prepare/spectroscopy',
            data=json.dumps(
                {'fileType': file_type, 'files': files}
            )
        )
        if not download_response:
            return None
        download_id = download_response['values'][0]

        self.dace.persist_file_on_disk(
            api_name=self.__OBS_API,
            obs_type='spectroscopy',
            download_id=download_id,
            output_directory=output_directory,
            output_filename=output_filename
        )

    def get_timeseries(self, target: str,
                       sorted_by_instrument: Optional[bool] = True,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Retrieve the spectroscopy time series data for a specified target in the chosen format.

        All available formats are defined in this section (see :doc:`output_format`).

        :param target: The target to retrieve data from.
        :type target: str
        :param sorted_by_instrument: Application of the instrument sorting
        :type sorted_by_instrument: Optional[bool]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.spectroscopy import Spectroscopy
        >>> target_to_search = "C15-0734"
        >>> values = Spectroscopy.get_timeseries(target=target_to_search)

        """
        spectroscopy_data = self.dace.request_get(
            api_name=self.__OBS_API,
            endpoint=f'observation/radialVelocities/{target}'
        )
        if not sorted_by_instrument:
            return self.dace.transform_to_format(spectroscopy_data, output_format=output_format)
        else:
            transformed_data = self.dace.transform_to_format(spectroscopy_data, output_format='numpy')
            return self.dace.order_spectroscopy_data_by_instruments(transformed_data)


Spectroscopy: SpectroscopyClass = SpectroscopyClass()
"""Spectroscopy instance"""



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.coordinates import SkyCoord, Angle
from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

CHEOPS_DEFAULT_LIMIT = 10000


class CheopsClass:
    """
    The cheops class.
    Use to retrieve data from the cheops module.

    **A cheops instance is already provided, to use it :**
    """
    __ACCEPTED_FILE_TYPES = ['lightcurves', 'images', 'reports', 'full', 'sub', 'all']
    __ACCEPTED_CATALOGS = ['planet', 'stellar']

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable cheops object which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.cheops import CheopsClass
        >>> cheops_instance = CheopsClass()

        """
        # Logging configuration
        self.__CHEOPS_API = 'cheops-webapp'
        self.log = logging.getLogger("Cheops")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = CHEOPS_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the cheops database to retrieve available visits in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.cheops import Cheops
        >>> values = Cheops.query_database()
        """

        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__CHEOPS_API,
                endpoint='search',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            ), output_format=output_format)

    def query_catalog(self,
                      catalog: str,
                      limit: Optional[int] = CHEOPS_DEFAULT_LIMIT,
                      filters: Optional[dict] = None,
                      sort: Optional[dict] = None,
                      output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the cheops, either stellar or planet, catalogs.

        Available catalogs are [ 'planet', 'stellar' ].

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param catalog: The catalog name
        :type catalog: str
        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.cheops import Cheops
        >>> catalog_to_search = 'planet'
        >>> values = Cheops.query_catalog(catalog_to_search)
        """

        if catalog not in self.__ACCEPTED_CATALOGS:
            raise ValueError('catalog must be one of these values : ' + ','.join(self.__ACCEPTED_CATALOGS))

        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__CHEOPS_API,
                endpoint=f'catalog/{catalog}',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }), output_format=output_format)

    def query_region(self,
                     sky_coord: SkyCoord,
                     angle: Angle,
                     limit: Optional[int] = CHEOPS_DEFAULT_LIMIT,
                     filters: Optional[dict] = None,
                     output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query a region, based on SkyCoord and Angle objects, in the Cheops database and retrieve data in the chosen
        format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param sky_coord: Sky coordinates object from the astropy module
        :type sky_coord: SkyCoord
        :param angle: Angle object from the astropy module
        :type angle: Angle
        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict


        >>> from dace_query.cheops import Cheops
        >>> from astropy.coordinates import SkyCoord, Angle
        >>> sky_coord, angle = SkyCoord("22h23m29s", "+32d27m34s", frame='icrs'), Angle('0.045d')
        >>> values = Cheops.query_region(sky_coord=sky_coord, angle=angle)

        """

        coordinate_filter_dict = self.dace.transform_coordinates_to_dict(sky_coord, angle)
        filters_with_coordinates = {}
        if filters is not None:
            filters_with_coordinates.update(filters)
        filters_with_coordinates.update(coordinate_filter_dict)
        return self.query_database(limit=limit, filters=filters_with_coordinates, output_format=output_format)

    def get_lightcurve(self,
                       target: str,
                       aperture: Optional[str] = 'default',
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Get the photometry data (from Cheops) in the chosen format for a specified target.

        Aperture types available are [ 'default', 'optimal, 'rinf', 'rsup' ].

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param target: The target to retrieve light curve from
        :type target: str
        :param aperture: Aperture type
        :type aperture: Optional[str]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: The desired data in the chosen output format
        :type output_format: Optional[str]
        :return: The desired data in the chosen format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.cheops import Cheops
        >>> values = Cheops.get_lightcurve('WASP-8')
        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        # a = 'photometry/' + target + '?aperture=' + aperture + '&filters=' +
        # self.dace.transform_dict_to_encoded_json(filters) + '&sort=' + self.dace.transform_dict_to_encoded_json(sort)

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__CHEOPS_API,
                endpoint=f'photometry/{target}',
                params={
                    'aperture': str(aperture),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }), output_format=output_format
        )

    def download(self,
                 file_type: str,
                 filters: Optional[dict] = None,
                 output_directory: Optional[str] = None,
                 output_filename: Optional[str] = None) -> None:
        """
        Download CHEOPS products (FITS, PDF,...) for specific visits and save it locally depending on the specified
        arguments.

        Filters can be applied to the query via named arguments (see :doc:`query_options`).

        File types available are [ 'lightcurves', 'images', 'reports', 'full', 'sub', 'all' ].

        All available formats are defined in this section (see :doc:`output_format`).

        :param file_type: The type of files to download
        :type file_type: str
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.cheops import Cheops
        >>> filters_to_use = {'file_key': {'contains': 'CH_PR300001_TG000301_V0000'}}
        >>> # Cheops.download('all', filters_to_use, output_directory='/tmp', output_filename='cheops.tar.gz')

        """
        if file_type not in self.__ACCEPTED_FILE_TYPES:
            raise ValueError('file_type must be one of these values : ' + ','.join(self.__ACCEPTED_FILE_TYPES))
        if filters is None:
            filters = {}

        cheops_data = self.query_database(filters=filters, output_format='dict')

        files = []
        if 'file_rootpath' in cheops_data:
            files = cheops_data['file_rootpath']

        download_id = self.dace.request_post(
            api_name=self.__CHEOPS_API,
            endpoint='download',
            data=json.dumps({'fileType': file_type, 'files': files})
        )
        if not download_id:
            return None

        self.dace.persist_file_on_disk(
            api_name=self.__CHEOPS_API,
            obs_type='photometry',
            download_id=download_id['key'],
            output_directory=output_directory,
            output_filename=output_filename
        )

    def download_diagnostic_movie(self,
                                  file_key: str,
                                  aperture: Optional[str] = 'default',
                                  output_directory: Optional[str] = None,
                                  output_filename: Optional[str] = None) -> None:
        """
        Download diagnostic movie for a Cheops file_key.

        Aperture types available are [ 'default', 'optimal, 'rinf', 'rsup' ].

        :param file_key: The cheops visit file key
        :type file_key: str
        :param aperture: Apertures types
        :type aperture: Optional[str]
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename:  The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.cheops import Cheops
        >>> # Cheops.download_diagnostic_movie(file_key='CH_PR100018_TG027204_V0200', output_directory='/tmp', output_filename='cheops_movie.mp4')

        """
        self.dace.download_file(
            api_name=self.__CHEOPS_API,
            endpoint=f'diagnosticMovie/{file_key}',
            params={
                'aperture': str(aperture)
            },
            output_directory=output_directory,
            output_filename=output_filename
        )


Cheops: CheopsClass = CheopsClass()
"""
Cheops instance
"""



from __future__ import annotations

import logging
from typing import Optional, Union

from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

MONITORING_DEFAULT_LIMIT = 10000


class MonitoringClass:
    """
    The monitoring class.
    Use to retrieve data from the monitoring module.

    **A monitoring instance is already provided, to use it :**

    >>> from dace_query.monitoring import Monitoring

    """
    __MONITORING_API = 'monitoring-webapp'

    def __init__(self, dace_instance: DaceClass = None):
        """
        Create a configurable monitoring object which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.monitoring import MonitoringClass
        >>> monitoring_instance = MonitoringClass()
        """

        # Logging configuration
        self.log = logging.getLogger("Monitoring")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_transfer_by_night(self,
                                instrument: str, pipeline: str, night: str,
                                output_format: Optional[str] = None
                                ) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the monitoring database to retrieve data in the chosen format for the specified instrument, night and
        pipeline.

        All available formats are defined in this section (see :doc:`output_format`).

        Available pipeline names are:
            - ``TRANSFER``: From remote archive to Geneva archive (Transfer)
            - ``GENEVA``: From reduction in Geneva to DACE import (Reduction and import)
            - ``FULL``: From remote archive to DACE import (Transfer, reduction and import)

        The supported combinations are indicated below and described as ``(instrument name; date of the night; pipeline name)`` :

            - HARPS, NIGHT, [ TRANSFER, GENEVA, FULL ]
            - ESPRESSO, NIGHT, [ TRANSFER, GENEVA, FULL ]
            - CORALIE14, NIGHT, [ TRANSFER, GENEVA, FULL ]
            - ECAM, NIGHT, [ TRANSFER ]
            - HARPN, NIGHT, [ TRANSFER ]

        :param instrument: The instrument name
        :type instrument: str
        :param pipeline: The pipeline name
        :type pipeline: str
        :param night: The date of the night
        :type night: str
        :param output_format: Type of data returns
        :type output_format: Optional[strs]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.monitoring import Monitoring
        >>> values = Monitoring.query_transfer_by_night(instrument='HARPS', pipeline='FULL', night='2022-11-08')
        """

        complete_name = f"{instrument}_{pipeline}".upper()

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__MONITORING_API,
                endpoint=f'monitoring/{complete_name}/date/{night}'),
            output_format=output_format
        )

    def query_transfer_by_period(self, instrument: str, pipeline: str, period: tuple[str, str],
                                 output_format: Optional[str] = None
                                 ) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the monitoring database ot retrieve data in the chosen format for the specified instrument, period and
        period.

        All available formats are defined in this section (see :doc:`output_format`).


        Available pipeline names are:
            - ``TRANSFER``: From remote archive to Geneva archive (Transfer)
            - ``GENEVA``: From reduction in Geneva to DACE import (Reduction and import)
            - ``FULL``: From remote archive to DACE import (Transfer, reduction and import)

        The supported combinations are indicated below and described as ``(instrument name; period dates; pipeline name)``:

            - HARPS, PERIOD, [ TRANSFER, GENEVA, FULL ]
            - ESPRESSO, PERIOD, [TRANSFER, GENEVA, FULL]
            - CORALIE14, PERIOD, [TRANSFER, GENEVA, FULL]
            - ECAM, PERIOD, [ TRANSFER ]
            - HARPN, PERIOD, [ GENEVA ]

        :param instrument: The instrument name
        :type instrument: str
        :param pipeline: The pipeline name
        :type pipeline: str
        :param period: The periods
        :type period: tuple[str, str]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.monitoring import Monitoring
        >>> values = Monitoring.query_transfer_by_period(instrument='HARPS', pipeline='FULL', period=('2022-11-07', '2022-11-09'))
        """

        complete_name = f"{instrument}_{pipeline}".upper()

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__MONITORING_API,
                endpoint=f'monitoring/{complete_name}/period/{period[0]}/{period[1]}'),
            output_format=output_format)

    def query_transfer_by_program(self, instrument: str, pipeline: str, program: str,
                                  output_format: Optional[str] = None
                                  ) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the monitoring database to retrieve data in the chosen format for the specified instrument, pipeline and
        program.

        All available formats are defined in this section (see :doc:`output_format`).


        Available pipeline names are:
            - ``TRANSFER``: From remote archive to Geneva archive (Transfer)
            - ``GENEVA``: From reduction in Geneva to DACE import (Reduction and import)
            - ``FULL``: From remote archive to DACE import (Transfer, reduction and import)


        The supported combinations are indicated below and described as ``(instrument name; program name; pipeline name)``:

            - HARPS, PROGRAM, [ TRANSFER, FULL ]
            - ESPRESSO, PROGRAM, [ TRANSFER, FULL ]


        :param instrument: The instrument name
        :type instrument: str
        :param pipeline: The pipeline name
        :type pipeline: str
        :param program: The program name
        :type program: str
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.monitoring import Monitoring
        >>> values = Monitoring.query_transfer_by_program(instrument='ESPRESSO', pipeline='TRANSFER', program='110.245W.001')

        """

        complete_name = f"{instrument}_{pipeline}".upper()

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__MONITORING_API,
                endpoint=f'monitoring/{complete_name}/program/{program}'),
            output_format=output_format)

    def query_transfer_by_target(self,
                                 instrument: str, pipeline: str, target: str,
                                 output_format: Optional[str] = None
                                 ):
        """
        Query the monitoring database to retrieve data in the chosen format for the specified instrument, pipeline and
        target.

        All available formats are defined in this section (see :doc:`output_format`).


        Available pipeline names are:
            - ``TRANSFER``: From remote archive to Geneva archive (Transfer)
            - ``GENEVA``: From reduction in Geneva to DACE import (Reduction and import)
            - ``FULL``: From remote archive to DACE import (Transfer, reduction and import)

        The supported combinations are indicated below and described as ``(instrument name; target name; pipeline name)``:

            - HARPS, TARGET, [ TRANSFER, FULL ]
            - ESPRESSO, TARGET, [ TRANSFER, FULL ]

        :param instrument: The instrument name
        :type instrument: str
        :param pipeline: The pipeline name
        :type pipeline: str
        :param target: The target name
        :type target: str
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.monitoring import Monitoring
        >>> values = Monitoring.query_transfer_by_target(instrument='ESPRESSO', pipeline='TRANSFER', target='L 513-23')
        """

        complete_name = f"{instrument}_{pipeline}".upper()

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__MONITORING_API,
                endpoint=f'monitoring/{complete_name}/target/{target}'),
            output_format=output_format
        )


Monitoring: MonitoringClass = MonitoringClass()
"""
Monitoring instance
"""



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

OPENDATA_DEFAULT_LIMIT = 10000


class OpenDataClass:
    """
    The opendata class.
    Use to retrieve data from the opendata module.

    **An opendata instance is already provided, to use it:**

    >>> from dace_query.opendata import OpenData
    """

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable opendata object which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.opendata import OpenDataClass
        >>> opendata_instance = OpenDataClass()
        """

        self.__OPEN_DATA_API = 'open-webapp'
        self.__OPENDATA_AVAILABLE_FILE_TYPES = ['readme', 'archive']
        self.__ADS_URL = 'https://ui.adsabs.harvard.edu/abs/'
        self.__DOI_URL = 'https://doi.org/'

        # Logging configuration
        self.log = logging.getLogger("OpenData")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = OPENDATA_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the open database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.opendata import OpenData
        >>> values =  OpenData.query_database()
        """

        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        data = self.dace.parse_parameters(
            self.dace.request_get(
                api_name=self.__OPEN_DATA_API,
                endpoint='publication/search',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            )
        )

        data['ads_link'] = [f'{self.__ADS_URL}{bibcode}' for bibcode in data['pub_bibcode']]
        data['doi_link'] = [f'{self.__DOI_URL}{doi}' for doi in data['pub_doi']]
        data['data_external_repositories'] = [json.loads(data_external_repositories) for data_external_repositories in
                                              data['data_external_repositories']]

        # Convert list of major into list of boolean values
        data['pub_major'] = list(
            map(lambda vector_major:
                list(
                    map(
                        lambda value: value.lower() == 'true', vector_major)),
                map(lambda record: record.split(','),
                    data['pub_major'])))

        return self.dace.convert_to_format(data, output_format=output_format)

    def download(self, dace_data_id: str,
                 file_type: str,
                 output_directory: Optional[str] = None,
                 output_filename: Optional[str] = None) -> None:
        """
        Download a publication specified by its dace_data_id, depending on the specified file type it downloads the archive
        or the readme file.

        Available file types are [ 'readme', 'archive' ].

        :param dace_data_id: The publication's data id
        :type bibcode: str
        :param file_type: The file to download
        :type file_type: str
        :param output_directory: The directory where files will be saved
        :type output_directory: Optional[str]
        :param output_filename: The filename for the download
        :type output_filename: Optional[str]
        :return: None

        >>> from dace_query.opendata import OpenData
        >>> # OpenData.download('2019MNRAS.483.5534S', 'archive', output_directory='/tmp', output_filename='opendata.tar.gz')

        """

        if file_type not in self.__OPENDATA_AVAILABLE_FILE_TYPES:
            raise ValueError('file_type must be : ' + ','.join(self.__OPENDATA_AVAILABLE_FILE_TYPES))

        self.dace.download_file(
            api_name=self.__OPEN_DATA_API,
            endpoint=f'publication/{file_type}/{dace_data_id}',
            output_directory=output_directory,
            output_filename=output_filename
        )


OpenData: OpenDataClass = OpenDataClass()
"""OpenData instance"""



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.coordinates import SkyCoord, Angle
from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

PHOTOMETRY_DEFAULT_LIMIT = 10000


class PhotometryClass:
    """
    The photometry class.
    Use to retrieve data from the photometry database.

    **A photometry instance is already provided, to use it:**

    >>> from dace_query.photometry import Photometry
    """
    __ACCEPTED_FILE_TYPES = ['s1d', 's2d', 'ccf', 'bis', 'all']
    __OBS_API = 'obs-webapp'
    __OBSERVATION_ENDPOINT = __OBS_API + 'observation/'
    __PHOTOMETRY_ENDPOINT = __OBSERVATION_ENDPOINT + 'photometry/'

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable photometry object which uses a specified dace.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.photometry import PhotometryClass
        >>> photometry_instance = PhotometryClass()
        """
        # Logging configuration
        self.log = logging.getLogger("Photometry")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = PHOTOMETRY_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:

        """
        Query the photometry database to retrieve data in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.photometry import Photometry
        >>> values =  Photometry.query_database()

        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__OBS_API,
                endpoint='observation/search/photometry',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            ), output_format=output_format
        )

    def query_region(self,
                     sky_coord: SkyCoord,
                     angle: Angle,
                     limit: Optional[int] = PHOTOMETRY_DEFAULT_LIMIT,
                     filters: Optional[dict] = None,
                     output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query a region, based on SkyCoord and Angle objects, in the photometry database and retrieve data in the chosen
        format.

        Filters can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param sky_coord: Sky coordinates object from the astropy module
        :type sky_coord: SkyCoord
        :param angle: Angle object from the astropy module
        :type angle: Angle
        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.photometry import Photometry
        >>> from astropy.coordinates import SkyCoord, Angle
        >>> sky_coord, angle = SkyCoord("11h01m04s", "+04d29m10s", frame='icrs'), Angle('0.045d')
        >>> values = Photometry.query_region(sky_coord=sky_coord, angle=angle)
        """

        coordinate_filter_dict = self.dace.transform_coordinates_to_dict(sky_coord, angle)
        filters_with_coordinates = {}
        if filters is not None:
            filters_with_coordinates.update(filters)
        filters_with_coordinates.update(coordinate_filter_dict)
        return self.query_database(limit=limit, filters=filters_with_coordinates, output_format=output_format)

    def get_timeseries(self, target: str) -> list:
        """
        Retrieve photometry timeseries for a specified target.

        :param target: The target to retrieve data from
        :type target: str
        :return: The desired data
        :rtype: list

        >>> from dace_query.photometry import Photometry
        >>> target_to_search = "EPIC201750173"
        >>> values = Photometry.get_timeseries(target=target_to_search)
        """
        result = self.dace.request_get(
            api_name=self.__OBS_API,
            endpoint=f'observation/photometry/{target}'
        )
        if not result:
            return None
        return result['observations']


Photometry: PhotometryClass = PhotometryClass()
"""A photometry instance"""



from __future__ import annotations

import json
import logging
from typing import Union, Optional

from astropy.table import Table
from numpy import ndarray
from pandas import DataFrame

from dace_query import Dace, DaceClass

POPULATION_DEFAULT_LIMIT = 10000


class PopulationClass:
    """
    The population class.
    Use to retrieve data from the population module.

    **A population instance is already provided, to use it:**

    >>> from dace_query.population import Population
    """

    SNAPSHOTS_DEFAULT_COLUMN = ['system_id', 'planet_id', 'total_mass', 'semi_major_axis']
    """Snapshot parameters retrieved by default"""
    SIMULATIONS_DEFAULT_COLUMN = ['total_mass', 'semi_major_axis']
    """Simulation parameters retrieved by default"""
    __POPULATION_API = 'evo-webapp'

    def __init__(self, dace_instance: Optional[DaceClass] = None):
        """
        Create a configurable population object which uses a specified dace instance.

        :param dace_instance: A dace object
        :type dace_instance: Optional[DaceClass]

        >>> from dace_query.population import PopulationClass
        >>> population_instance = PopulationClass()
        """
        # Logging configuration
        self.log = logging.getLogger("Population")
        self.log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.log.addHandler(ch)

        if dace_instance is None:
            self.dace = Dace
        elif isinstance(dace_instance, DaceClass):
            self.dace = dace_instance
        else:
            raise Exception("Dace instance is not valid")

    def query_database(self,
                       limit: Optional[int] = POPULATION_DEFAULT_LIMIT,
                       filters: Optional[dict] = None,
                       sort: Optional[dict] = None,
                       output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Query the population database to retrieve data (population descriptions) in the chosen format.

        Filters and sorting order can be applied to the query via named arguments (see :doc:`query_options`).

        All available formats are defined in this section (see :doc:`output_format`).

        :param limit: Maximum number of rows to return
        :type limit: Optional[int]
        :param filters: Filters to apply to the query
        :type filters: Optional[dict]
        :param sort: Sort order to apply to the query
        :type sort: Optional[dict]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.population import Population
        >>> values = Population.query_database()

        """
        if filters is None:
            filters = {}
        if sort is None:
            sort = {}

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__POPULATION_API,
                endpoint='population/search',
                params={
                    'limit': str(limit),
                    'filters': json.dumps(filters),
                    'sort': json.dumps(sort)
                }
            ), output_format=output_format
        )

    def get_columns(self, population_id: str, output_format: Optional[str] = None) \
            -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Get the available columns for the specified population.

        All available formats are defined in this section (see :doc:`output_format`).

        :param population_id: The population id
        :type population_id: str
        :param output_format: Type of data returns
        :type output_format: str
        :return: The desired data in the chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.population import Population
        >>> population_to_search = 'ng96'
        >>> values = Population.get_columns('ng96')
        """

        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__POPULATION_API,
                endpoint=f'population/{population_id}/variables'
            ), output_format=output_format
        )

    def get_snapshots(self,
                      population_id: str,
                      years: str,
                      columns: Optional[list[str]] = None,
                      output_format: Optional[str] = None) \
            -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Get snapshots data for a specific population at a specific age.

        All available formats are defined in this section (see :doc:`output_format`).

        :param population_id: The population id to get snapshots from
        :type population_id: str
        :param years: The specific age of the snapshot
        :type years: str
        :param columns: A list of parameters to retrieve
        :type columns: Optional[list[str]]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: The desired data in chosen output format
        :rtype: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.population import Population
        >>> population_id = 'ng96'
        >>> years = '5000000'
        >>> columns_to_retrieve = ['system_id', 'planet_id', 'total_mass']
        >>> values = Population.get_snapshots(population_id=population_id, years=years, columns=columns_to_retrieve)
        """
        if columns is None:
            columns = self.SNAPSHOTS_DEFAULT_COLUMN
        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__POPULATION_API,
                endpoint=f'population/{population_id}/snapshots/{str(years)}',
                params={'col': columns}
            ), output_format=output_format
        )

    @staticmethod
    def get_snapshot_ages() -> list[str]:
        """
        Use to get all snapshot ages.

        :return: All existing ages
        :rtype: list[str]

        >>> from dace_query.population import Population
        >>> ages = Population.get_snapshot_ages()
        """

        snapshot_ages = []
        for exp in range(5, 10):
            for base in range(1, 10):
                age = base * 10.0 ** exp
                snapshot_ages.append(age)
        snapshot_ages.append(1E10)
        snapshot_ages = list(map(lambda v: str(int(v)), snapshot_ages))
        return snapshot_ages

    def get_track(self,
                  population_id: str,
                  system_id: int,
                  planet_id: int,
                  columns: Optional[list[str]] = None,
                  output_format: Optional[str] = None) -> Union[dict[str, ndarray], DataFrame, Table, dict]:
        """
        Retrieve tracks for a specific population, system id and planet id.

        All available formats are defined in this section (see :doc:`output_format`).

        :param population_id: The population id to retrieve tracks from
        :type population_id: str
        :param system_id: The system id
        :type system_id: int
        :param planet_id: The planet id
        :type planet_id: int
        :param columns: The parameters to retrieve
        :type columns: Optional[list[str]]
        :param output_format: Type of data returns
        :type output_format: Optional[str]
        :return: dict[str, ndarray] or DataFrame or Table or dict

        >>> from dace_query.population import Population
        >>> pop_id, planet_id, system_id = 'ng96', 1, 1
        >>> parameters_to_retrieve = ['time_yr', 'total_mass']
        >>> values = Population.get_track(population_id=pop_id, system_id=system_id, planet_id=planet_id, columns=parameters_to_retrieve)

        """
        if columns is None:
            columns = self.SIMULATIONS_DEFAULT_COLUMN
        return self.dace.transform_to_format(
            self.dace.request_get(
                api_name=self.__POPULATION_API,
                endpoint=f'population/{population_id}/{system_id}/{planet_id}/simulations',
                params={'col': columns}
            ), output_format=output_format
        )


Population: PopulationClass = PopulationClass()
"""Population instance"""



from __future__ import annotations

import configparser
import json
import logging
import re
import urllib.parse
from collections import defaultdict
from functools import partial
from pathlib import Path
from typing import Optional, Union

import numpy as np
import requests
from astropy.coordinates import SkyCoord, Angle
from astropy.table import Table
from pandas import DataFrame
from requests import RequestException, HTTPError

from dace_query.__version__ import __version__, __title__, __py_version__

COORDINATES_DB_COLUMN = 'obj_pos_coordinates_hms_dms'

MB_SIZE = 1048576


class NoDataException(Exception):
    """Raised when no data are provided"""


class DaceClass:
    """
    The dace class.
    When initializing an instance with this class, it loads the user .dacerc and config.ini files to authentify the
    current user and define which endpoints to use.

    The **.dacerc** file, (**you have to create it**), located by default in the home directory (~/.dacerc) and in TOML
    format, defines a user section with a key-value pair specifying the user's API key (see below).

    .. code-block:: cfg

        [user]
        key = apiKey:<xxx-xxx-xxx>

    To obtain an API key:

    1.  Login on dace (https://dace.unige.ch)
    2.  Go to the user profile
    3.  Click on [Generate a new API key]

    The **config.ini** file, in CFG format and already included in the dace package, defines default DACE endpoints
    to use (see below).

    .. code-block:: cfg

        [api]

        evo-webapp = https://evo-webapp.obsuksprd2.unige.ch/
        exo-webapp = https://exo-webapp.obsuksprd2.unige.ch/
        lossy-webapp = https://lossy-webapp.obsuksprd2.unige.ch/
        obs-webapp = https://obs-webapp.obsuksprd2.unige.ch/
        opa-webapp = https://opa-webapp.obsuksprd2.unige.ch/
        open-webapp = https://od-webapp.obsuksprd2.unige.ch/
        tess-webapp = https://tess-webapp.obsuksprd2.unige.ch/
        cheops-webapp = https://cheops-webapp.obsuksprd2.unige.ch/
        monitoring-webapp = https://pipe-webapp.obsuksprd2.unige.ch/

    **A dace instance is already provided, to use it :**

    >>> from dace_query import Dace


    """

    def __init__(self, dace_rc_config_path: Optional[Path] = None, config_path: Optional[Path] = None):
        """
        Create a configurable dace object which loads the user's .dacerc and the config file specified in arguments.

        :param dace_rc_config_path: The .dacerc filepath, used to authentify the user.
        :type dace_rc_config_path: Optional[Path]
        :param config_path: The config.ini filepath, defines which DACE endpoints to use.
        :type config_path: Optional[Path]

        >>> from dace_query import DaceClass
        >>> from pathlib import Path
        >>> dace_instance = DaceClass(dace_rc_config_path=Path.home(), config_path=Path('config.ini'))
        >>> type(DaceClass())
        <class 'dace.dace.DaceClass'>

        """
        # Logging configuration
        logger = logging.getLogger('dace')
        logger.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        ch.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s'))
        logger.addHandler(ch)
        self.log = logger

        # Get dacerc and endpoints location
        dace_rc_config_path = Path(Path.home(), '.dacerc') if dace_rc_config_path is None else Path(dace_rc_config_path)
        endpoints_path = Path(Path(__file__).parent.resolve(), 'config.ini') if config_path is None else Path(
            config_path)

        self.__dace_rc_config = None
        # Load the dace resource file <.dacerc>
        if dace_rc_config_path.exists() and dace_rc_config_path.is_file():
            dace_rc_config = configparser.ConfigParser()
            dace_rc_config.read(dace_rc_config_path)
            self.__dace_rc_config = dace_rc_config
        else:
            self.log.warning(
                "File .dacerc not found. You are requesting data in public mode. "
                "To change this behaviour, create a .dacerc file in your home directory and fill it with your API key. "
                "More infos on https://dace.unige.ch")

        # Load the endpoint configuration file <config.ini>
        self.__cfg = None
        if endpoints_path.exists() and endpoints_path.is_file():
            endpoints_config = configparser.ConfigParser()
            endpoints_config.read(endpoints_path)
            self.__cfg = endpoints_config

    @staticmethod
    def transform_dict_to_encoded_json(dict_to_transform: Union[set, dict]) -> str:
        """Internal stuff"""
        return urllib.parse.quote_plus(json.dumps(dict_to_transform))

    @staticmethod
    def transform_coordinates_to_dict(sky_coord: SkyCoord, angle: Angle) -> dict:
        """Internal stuff"""
        return {COORDINATES_DB_COLUMN: {'ra': sky_coord.ra.degree, 'dec': sky_coord.dec.degree,
                                        'radius': angle.degree}}

    def transform_to_format(self, json_data: dict, output_format: Optional[str] = None):
        """Internal stuff"""
        data = self.parse_parameters(json_data)
        return self.convert_to_format(data, output_format)

    def parse_parameters(self, json_data: dict) -> dict[str, list]:
        """Internal stuff"""
        """
        Internally DACE data are provided using protobuf. The format is a list of parameters. Here we parse
        these data to give to the user something more readable and ignore the internal stuff
        """
        data = defaultdict(list)
        if 'parameters' not in json_data:
            return data
        parameters = json_data.get('parameters')
        for parameter in parameters:
            variable_name = parameter.get('variableName')
            double_values = parameter.get('doubleValues')
            # W/A for 'NaN' values
            if double_values is not None:
                double_values = list(map(float, double_values))
            float_values = parameter.get('floatValues')
            # W/A for 'NaN' values
            if float_values is not None:
                float_values = list(map(float, float_values))
            int_values = parameter.get('intValues')
            string_values = parameter.get('stringValues')
            bool_values = parameter.get('boolValues')
            occurrences = parameter.get('occurrences')

            # Only one type of values can be present. So we look for the next occurrence not None. Prevent not found
            # with an empty list to avoid having StopIteration exception
            values = next(
                (values_list for values_list in [double_values, float_values, int_values, string_values, bool_values] if
                 values_list is not None), [])
            if occurrences:
                data[variable_name].extend(self.__transform_values_with_occurrences(values, occurrences))
            else:
                data[variable_name].extend(values)

            error_values = parameter.get('minErrorValues')  # min or max is symmetric
            if error_values is not None:
                if occurrences:
                    data[variable_name + '_err'].extend(
                        self.__transform_values_with_occurrences(error_values, occurrences))
                else:
                    data[variable_name + '_err'].extend(error_values)
        return data

    @staticmethod
    def convert_to_format(data: dict, output_format: Optional[str]) -> Union[
        dict[str, np.ndarray], DataFrame, Table, dict]:
        """Internal stuff"""

        if output_format == 'pandas':
            return DataFrame.from_dict(data)
        elif output_format == 'astropy_table':
            return Table(data)
        elif output_format == 'dict':
            return data
        else:  # or output_format='numpy'
            np_data = {}
            for key, values in data.items():
                if any(map(lambda value: type(value) == list, values)):
                    np_data[key] = np.array(values, dtype='object')
                else:
                    np_data[key] = np.array(values)
            return np_data

    def persist_file_on_disk(self, api_name: str, obs_type: str, download_id: str,
                             params: Optional[dict] = None,
                             output_directory: Optional[str] = None,
                             output_filename: Optional[str] = None) -> None:
        """Internal stuff"""
        self.download_file(
            api_name=api_name,
            endpoint=f'download/{obs_type}/{download_id}',
            params=params,
            output_directory=output_directory,
            output_filename=output_filename
        )

    def request_get(self, api_name: str, endpoint: str, params: Optional[dict] = None,
                    raw_response: Optional[bool] = False) -> Union[bytes, dict]:
        """Internal stuff"""

        """
        This method does an HTTP get to DACE backend. If an apiKey has been found, it will be added in HTTP header
        :param endpoint: the DACE endpoint you want to query
        :return: the Json response containing data
        """
        headers = self.__prepare_request(raw_response)

        host = self.__cfg['api'][api_name] + endpoint
        try:
            response = requests.get(host, headers=headers, params=params)
            response.raise_for_status()

            if response.ok:
                if raw_response:
                    return response.content
                else:
                    return response.json()
            else:
                self.log.error("Status code %s when calling %s", response.status_code, host)
                raise RequestException
        except HTTPError as err_h:
            return self.__manage_http_errors(err_h)
        except RequestException as e:
            raise RequestException('Problem when calling {}'.format(host)) from e

    def request_post(self, api_name: str, endpoint: str,
                     json_data: Optional[dict] = None,
                     data: Optional[str] = None,
                     params: Optional[dict] = None) -> dict:

        """Internal stuff"""

        headers = self.__prepare_request()
        host = self.__cfg['api'][api_name] + endpoint

        try:
            response = requests.post(host, headers=headers, json=json_data, data=data, params=params)
            response.raise_for_status()
            if response.ok:
                return response.json()
            else:
                self.log.error("Status code %s when calling %s", response.status_code, host)
                raise RequestException
        except HTTPError as err_h:
            return self.__manage_http_errors(err_h)
        except RequestException as e:
            raise RequestException('Problem when calling {}'.format(host)) from e

    def download_file(self,
                      api_name: str,
                      endpoint: str,
                      params: Optional[dict] = None,
                      output_directory: Optional[str] = None,
                      output_filename: Optional[str] = None) -> None:
        """Internal stuff"""
        try:
            if output_directory is None:
                output_directory = Path.home()
            with requests.get(self.__cfg['api'][api_name] + endpoint,
                              params=params,
                              headers=self.__prepare_request(True),
                              stream=True) as response:
                response.raise_for_status()
                if output_filename is None:
                    output_filename = re.sub("attachment;\\s*filename\\s*=\\s*", '',
                                             response.headers['content-disposition']).replace('"', '')
                    if output_filename is None:
                        raise ValueError('Missing content-disposition. Please contact DACE support')
                output_full_file_path = Path(output_directory, output_filename)
                self.log.info("Downloading file on location : %s", output_full_file_path)
                self.write_stream(output_full_file_path, response)
                self.log.info('File downloaded on location : %s', output_full_file_path)
        except HTTPError as err_h:
            if err_h.response.status_code == 404:
                self.log.error('The file is not found on DACE')
            else:
                self.__manage_http_errors(err_h)

    @staticmethod
    def write_stream(output_filename: Union[Path, str], response: requests.Response) -> None:
        """Internal stuff"""

        with open(output_filename, 'wb') as f:
            chunk_total_size = 0
            for chunk in response.iter_content(chunk_size=8192):
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)
                    chunk_total_size += 8192
                    if chunk_total_size % MB_SIZE == 0:
                        print("\r Download : " + str(chunk_total_size // MB_SIZE) + " MB", end="")
        print("\nDownload done")

    def __manage_http_errors(self, err_h) -> dict:
        """Internal stuff"""
        status_code = err_h.response.status_code
        if status_code == 404:
            self.log.error('No data found.')
        elif status_code == 401:
            self.log.error('Not authorized. You need to be logged on to access these data.')
        elif status_code == 403:
            self.log.error('Forbidden. You do not have the permission to access these data.')
        elif status_code == 405:
            self.log.error('Not Allowed. This method is deprecated. Please refer to the documentation.')
        else:
            self.log.error('Http error : ' + str(err_h))
            self.log.error('Please contact DACE support')
        return {}

    def __prepare_request(self, raw_response: Optional[bool] = False) -> dict:
        """Internal stuff"""
        headers = {'Accept': 'application/octet-stream'} if raw_response else {'Accept': 'application/json'}
        if self.__dace_rc_config is not None:
            headers['Authorization'] = self.__dace_rc_config['user']['key']
        headers['User-Agent'] = '/'.join([__title__, __version__, __py_version__])
        return headers

    @staticmethod
    def order_spectroscopy_data_by_instruments(data: dict[str, np.ndarray]) -> dict:
        """Internal stuff"""
        instruments_names = data.pop('ins_name', None)
        instruments_modes = data.pop('ins_mode', None)
        drs_versions = data.pop('drs_version', None)
        bib_codes = data.pop('pub_bibcode', None)

        data_by_instrument = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list))))
        # Collect and group the data by ins_name / drs_version / ins_mode / parameter
        for i, ins_name in enumerate(instruments_names):
            for parameter, values in data.items():
                bibcode = bib_codes[i] if (bib_codes is not None and i < len(bib_codes)) else None
                drs_version = drs_versions[i] if (drs_versions is not None and i < len(drs_versions)) else None
                drs_or_bibcode = bibcode or drs_version or 'default'
                ins_mode = instruments_modes[i] if (
                        instruments_modes is not None and i < len(instruments_modes)) else 'default'
                # print('Value %s,  %s, %s' % (data[parameter][i], parameter, i))
                data_by_instrument[ins_name][drs_or_bibcode][ins_mode][parameter].append(data[parameter][i])

        numpy_data_by_instrument = defaultdict(
            lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(partial(np.ndarray, 0)))))

        for ins_key, drs_list in data_by_instrument.items():
            for drs_key, mode_list in drs_list.items():
                for mode_key, parameter_list in mode_list.items():
                    for parameter_key, values in parameter_list.items():
                        numpy_data_by_instrument[ins_key][drs_key][mode_key][parameter_key] = np.array(values)

        return numpy_data_by_instrument

    @staticmethod
    def __transform_values_with_occurrences(values: list, occurrences: dict) -> list:
        """Internal stuff"""
        full_vector = []
        for i, occurrence in enumerate(occurrences):
            for j in range(0, occurrence):
                full_vector.append(values[i])

        return full_vector


Dace = DaceClass()
"""Dace instance"""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dace_query',
 'dace_query.astrometry',
 'dace_query.atmosphericSpectroscopy',
 'dace_query.catalog',
 'dace_query.cheops',
 'dace_query.exoplanet',
 'dace_query.imaging',
 'dace_query.lossy',
 'dace_query.monitoring',
 'dace_query.opacity',
 'dace_query.opendata',
 'dace_query.photometry',
 'dace_query.population',
 'dace_query.spectroscopy',
 'dace_query.sun',
 'dace_query.target',
 'dace_query.tess']

package_data = \
{'': ['*']}

install_requires = \
['astropy>=5.1.1,<6.0.0',
 'numpy>=1.23.4,<2.0.0',
 'pandas>=1.5.1,<2.0.0',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'dace-query',
    'version': '1.0.1',
    'description': 'The dace-query lets easily query DACE and access public and private data using a simple utility tool.',
    'long_description': 'dace-query\n##########\n\nDescription\n***********\nThe dace-query lets easily query DACE and access public and private data using a simple utility tool.\n\nInstallation\n************\n\nThe dace-query is available on PyPi and can be installed using `pip <https://pypi.org/project/pip/>`_ or `conda <https://www.anaconda.com>`_:\n\n\n.. code-block:: bash\n\n    # Install using pip\n    pip install dace-query\n\n    # Update using pip\n    pip install dace-query --upgrade\n\n.. code-block:: bash\n\n    # Install using conda\n    # First, create a conda environment\n    conda create -n <env_name> python=3.9 # python version equal or upper than 3.9\n    # Second, activate the environment\n    conda activate <env_name>\n    # Finally, install using anaconda pip\n    pip install dace-query\n\nMake sure the API is installed correctly :\n\n.. code-block:: python\n\n    # Import dace\n    import dace_query\n\n    # List content of the dace package\n    help(dace)\n\nAuthentication\n**************\n\nIn order to access the private data of DACE, an authentication system has been implemented.\nThis one works very simply, it just requires three things detailed in the following subsections:\n\n\n- A DACE account\n- An API key\n- A local .dacerc file\n\n\n.. _create-account:\n\n1. Create an account\n====================\nRegister on the `DACE web portal <https://dace.unige.ch/createAccount/>`_ with a university email address.\n\n.. _api-key:\n\n2. Generate the DACE API key\n============================\nTo obtain an API key:\n\n    1.  Login on DACE (https://dace.unige.ch)\n    2.  Go to the user profile\n    3.  Click on [Generate a new API key]\n    4.  Copy this new API key into the .dacerc file\n\n\n.. _dacerc:\n\n3. The .dacerc file\n===================\nThe **.dacerc** file, (**you have to create it**), located by default in the home directory (~/.dacerc) and in TOML\nformat, defines a user section with a key-value pair specifying the user\'s API key (see below).\n\n.. code-block:: cfg\n\n    [user]\n    key = apiKey:<xxx-xxx-xxx>\n\nFor example, if your API key is 12345678-1234-5678-1234-567812345678, then the .dacerc file will be :\n\n.. code-block:: cfg\n\n    [user]\n    key = apiKey:12345678-1234-5678-1234-567812345678\n\nTo create the .dacerc file on Linux or macOs, open a terminal window and type :\n\n.. code-block:: bash\n\n    printf \'[user]\\nkey = apiKey:%s\\n\' "your-api-key-here" > ~/.dacerc\n\nQuickstart\n**********\n\n.. code-block:: python\n\n    # Import the ready-to-use exoplanet instance\n    from dace_query.exoplanet import Exoplanet\n\n    # Retrieve data from the exoplanet database\n    result: dict = Exoplanet.query_database(limit=10, output_format=\'dict\')\n\n    # Get the planet names\n    planet_names: list = result.get(\'obj_id_catname\')\n\n    # Print the planet names\n    print(planet_names)\n\n\nFor more examples of uses, such as **filtering bad quality data** (see Usage examples)\n\nContact\n*******\n\nIn case of questions, proposals or problems, feel free to contact the `DACE support <mailto:dace-support@unige.ch>`_ .\n\nLinks\n*****\n* `DACE website <https://dace.unige.ch>`_\n',
    'author': 'dace-team',
    'author_email': 'dace-support@unige.ch',
    'maintainer': 'dace-team',
    'maintainer_email': 'dace-support@unige.ch',
    'url': 'https://dace.unige.ch/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
